# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class SecurityPolicy(pulumi.CustomResource):
    description: pulumi.Output[str]
    """
    An optional description of this rule. Max size is 64.
    """
    fingerprint: pulumi.Output[str]
    """
    Fingerprint of this resource.
    """
    name: pulumi.Output[str]
    """
    The name of the security policy.
    """
    project: pulumi.Output[str]
    """
    The project in which the resource belongs. If it
    is not provided, the provider project is used.
    """
    rules: pulumi.Output[list]
    """
    The set of rules that belong to this policy. There must always be a default
    rule (rule with priority 2147483647 and match "\*"). If no rules are provided when creating a
    security policy, a default rule with action "allow" will be added. Structure is documented below.

      * `action` (`str`) - Action to take when `match` matches the request. Valid values:
        * "allow" : allow access to target
        * "deny(status)" : deny access to target, returns the  HTTP response code specified (valid values are 403, 404 and 502)
      * `description` (`str`) - An optional description of this rule. Max size is 64.
      * `match` (`dict`) - A match condition that incoming traffic is evaluated against.
        If it evaluates to true, the corresponding `action` is enforced. Structure is documented below.
        * `config` (`dict`) - The configuration options available when specifying `versioned_expr`.
          This field must be specified if `versioned_expr` is specified and cannot be specified if `versioned_expr` is not specified.
          Structure is documented below.
          * `srcIpRanges` (`list`) - Set of IP addresses or ranges (IPV4 or IPV6) in CIDR notation
            to match against inbound traffic. There is a limit of 5 IP ranges per rule. A value of '\*' matches all IPs
            (can be used to override the default behavior).

        * `expr` (`dict`) - User defined CEVAL expression. A CEVAL expression is used to specify match criteria
          such as origin.ip, source.region_code and contents in the request header.
          Structure is documented below.
          * `expression` (`str`) - Textual representation of an expression in Common Expression Language syntax.
            The application context of the containing message determines which well-known feature set of CEL is supported.

        * `versionedExpr` (`str`) - Predefined rule expression. If this field is specified, `config` must also be specified.
          Available options:
          * SRC_IPS_V1: Must specify the corresponding `src_ip_ranges` field in `config`.

      * `preview` (`bool`) - When set to true, the `action` specified above is not enforced.
        Stackdriver logs for requests that trigger a preview action are annotated as such.
      * `priority` (`float`) - An unique positive integer indicating the priority of evaluation for a rule.
        Rules are evaluated from highest priority (lowest numerically) to lowest priority (highest numerically) in order.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    def __init__(__self__, resource_name, opts=None, description=None, name=None, project=None, rules=None, __props__=None, __name__=None, __opts__=None):
        """
        A Security Policy defines an IP blacklist or whitelist that protects load balanced Google Cloud services by denying or permitting traffic from specified IP ranges. For more information
        see the [official documentation](https://cloud.google.com/armor/docs/configure-security-policies)
        and the [API](https://cloud.google.com/compute/docs/reference/rest/beta/securityPolicies).

        ## Example Usage



        ```python
        import pulumi
        import pulumi_gcp as gcp

        policy = gcp.compute.SecurityPolicy("policy", rules=[
            {
                "action": "deny(403)",
                "description": "Deny access to IPs in 9.9.9.0/24",
                "match": {
                    "config": {
                        "srcIpRanges": ["9.9.9.0/24"],
                    },
                    "versionedExpr": "SRC_IPS_V1",
                },
                "priority": "1000",
            },
            {
                "action": "allow",
                "description": "default rule",
                "match": {
                    "config": {
                        "srcIpRanges": ["*"],
                    },
                    "versionedExpr": "SRC_IPS_V1",
                },
                "priority": "2147483647",
            },
        ])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: An optional description of this rule. Max size is 64.
        :param pulumi.Input[str] name: The name of the security policy.
        :param pulumi.Input[str] project: The project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[list] rules: The set of rules that belong to this policy. There must always be a default
               rule (rule with priority 2147483647 and match "\*"). If no rules are provided when creating a
               security policy, a default rule with action "allow" will be added. Structure is documented below.

        The **rules** object supports the following:

          * `action` (`pulumi.Input[str]`) - Action to take when `match` matches the request. Valid values:
            * "allow" : allow access to target
            * "deny(status)" : deny access to target, returns the  HTTP response code specified (valid values are 403, 404 and 502)
          * `description` (`pulumi.Input[str]`) - An optional description of this rule. Max size is 64.
          * `match` (`pulumi.Input[dict]`) - A match condition that incoming traffic is evaluated against.
            If it evaluates to true, the corresponding `action` is enforced. Structure is documented below.
            * `config` (`pulumi.Input[dict]`) - The configuration options available when specifying `versioned_expr`.
              This field must be specified if `versioned_expr` is specified and cannot be specified if `versioned_expr` is not specified.
              Structure is documented below.
              * `srcIpRanges` (`pulumi.Input[list]`) - Set of IP addresses or ranges (IPV4 or IPV6) in CIDR notation
                to match against inbound traffic. There is a limit of 5 IP ranges per rule. A value of '\*' matches all IPs
                (can be used to override the default behavior).

            * `expr` (`pulumi.Input[dict]`) - User defined CEVAL expression. A CEVAL expression is used to specify match criteria
              such as origin.ip, source.region_code and contents in the request header.
              Structure is documented below.
              * `expression` (`pulumi.Input[str]`) - Textual representation of an expression in Common Expression Language syntax.
                The application context of the containing message determines which well-known feature set of CEL is supported.

            * `versionedExpr` (`pulumi.Input[str]`) - Predefined rule expression. If this field is specified, `config` must also be specified.
              Available options:
              * SRC_IPS_V1: Must specify the corresponding `src_ip_ranges` field in `config`.

          * `preview` (`pulumi.Input[bool]`) - When set to true, the `action` specified above is not enforced.
            Stackdriver logs for requests that trigger a preview action are annotated as such.
          * `priority` (`pulumi.Input[float]`) - An unique positive integer indicating the priority of evaluation for a rule.
            Rules are evaluated from highest priority (lowest numerically) to lowest priority (highest numerically) in order.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['description'] = description
            __props__['name'] = name
            __props__['project'] = project
            __props__['rules'] = rules
            __props__['fingerprint'] = None
            __props__['self_link'] = None
        super(SecurityPolicy, __self__).__init__(
            'gcp:compute/securityPolicy:SecurityPolicy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, description=None, fingerprint=None, name=None, project=None, rules=None, self_link=None):
        """
        Get an existing SecurityPolicy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: An optional description of this rule. Max size is 64.
        :param pulumi.Input[str] fingerprint: Fingerprint of this resource.
        :param pulumi.Input[str] name: The name of the security policy.
        :param pulumi.Input[str] project: The project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[list] rules: The set of rules that belong to this policy. There must always be a default
               rule (rule with priority 2147483647 and match "\*"). If no rules are provided when creating a
               security policy, a default rule with action "allow" will be added. Structure is documented below.
        :param pulumi.Input[str] self_link: The URI of the created resource.

        The **rules** object supports the following:

          * `action` (`pulumi.Input[str]`) - Action to take when `match` matches the request. Valid values:
            * "allow" : allow access to target
            * "deny(status)" : deny access to target, returns the  HTTP response code specified (valid values are 403, 404 and 502)
          * `description` (`pulumi.Input[str]`) - An optional description of this rule. Max size is 64.
          * `match` (`pulumi.Input[dict]`) - A match condition that incoming traffic is evaluated against.
            If it evaluates to true, the corresponding `action` is enforced. Structure is documented below.
            * `config` (`pulumi.Input[dict]`) - The configuration options available when specifying `versioned_expr`.
              This field must be specified if `versioned_expr` is specified and cannot be specified if `versioned_expr` is not specified.
              Structure is documented below.
              * `srcIpRanges` (`pulumi.Input[list]`) - Set of IP addresses or ranges (IPV4 or IPV6) in CIDR notation
                to match against inbound traffic. There is a limit of 5 IP ranges per rule. A value of '\*' matches all IPs
                (can be used to override the default behavior).

            * `expr` (`pulumi.Input[dict]`) - User defined CEVAL expression. A CEVAL expression is used to specify match criteria
              such as origin.ip, source.region_code and contents in the request header.
              Structure is documented below.
              * `expression` (`pulumi.Input[str]`) - Textual representation of an expression in Common Expression Language syntax.
                The application context of the containing message determines which well-known feature set of CEL is supported.

            * `versionedExpr` (`pulumi.Input[str]`) - Predefined rule expression. If this field is specified, `config` must also be specified.
              Available options:
              * SRC_IPS_V1: Must specify the corresponding `src_ip_ranges` field in `config`.

          * `preview` (`pulumi.Input[bool]`) - When set to true, the `action` specified above is not enforced.
            Stackdriver logs for requests that trigger a preview action are annotated as such.
          * `priority` (`pulumi.Input[float]`) - An unique positive integer indicating the priority of evaluation for a rule.
            Rules are evaluated from highest priority (lowest numerically) to lowest priority (highest numerically) in order.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["description"] = description
        __props__["fingerprint"] = fingerprint
        __props__["name"] = name
        __props__["project"] = project
        __props__["rules"] = rules
        __props__["self_link"] = self_link
        return SecurityPolicy(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class TagTemplate(pulumi.CustomResource):
    display_name: pulumi.Output[str]
    """
    The display name for this template.
    """
    fields: pulumi.Output[list]
    """
    Set of tag template field IDs and the settings for the field. This set is an exhaustive list of the allowed fields. This set must contain at least one field and at most 500 fields.  Structure is documented below.

      * `display_name` (`str`) - The display name for this template.
      * `fieldId` (`str`) - The identifier for this object. Format specified above.
      * `isRequired` (`bool`) - Whether this is a required field. Defaults to false.
      * `name` (`str`) - -
        The resource name of the tag template field in URL format. Example: projects/{project_id}/locations/{location}/tagTemplates/{tagTemplateId}/fields/{field}
      * `order` (`float`) - The order of this field with respect to other fields in this tag template.
        A higher value indicates a more important field. The value can be negative.
        Multiple fields can have the same order, and field orders within a tag do not have to be sequential.
      * `type` (`dict`) - The type of value this tag field can contain.  Structure is documented below.
        * `enumType` (`dict`) - Represents an enum type.
          Exactly one of `primitive_type` or `enum_type` must be set  Structure is documented below.
          * `allowedValues` (`list`) - The set of allowed values for this enum. The display names of the
            values must be case-insensitively unique within this set. Currently,
            enum values can only be added to the list of allowed values. Deletion
            and renaming of enum values are not supported.
            Can have up to 500 allowed values.  Structure is documented below.
            * `display_name` (`str`) - The display name for this template.

        * `primitiveType` (`str`) - Represents primitive types - string, bool etc.
          Exactly one of `primitive_type` or `enum_type` must be set
    """
    force_delete: pulumi.Output[bool]
    """
    This confirms the deletion of any possible tags using this template. Must be set to true in order to delete the tag template.
    """
    name: pulumi.Output[str]
    """
    -
    The resource name of the tag template field in URL format. Example: projects/{project_id}/locations/{location}/tagTemplates/{tagTemplateId}/fields/{field}
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    Template location region.
    """
    tag_template_id: pulumi.Output[str]
    """
    The id of the tag template to create.
    """
    def __init__(__self__, resource_name, opts=None, display_name=None, fields=None, force_delete=None, project=None, region=None, tag_template_id=None, __props__=None, __name__=None, __opts__=None):
        """
        A tag template defines a tag, which can have one or more typed fields.
        The template is used to create and attach the tag to GCP resources.


        To get more information about TagTemplate, see:

        * [API documentation](https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.tagTemplates)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/data-catalog/docs)

        ## Example Usage

        ### Data Catalog Tag Template Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        basic_tag_template = gcp.datacatalog.TagTemplate("basicTagTemplate",
            display_name="Demo Tag Template",
            fields=[
                {
                    "display_name": "Source of data asset",
                    "fieldId": "source",
                    "isRequired": True,
                    "type": {
                        "primitiveType": "STRING",
                    },
                },
                {
                    "display_name": "Number of rows in the data asset",
                    "fieldId": "num_rows",
                    "type": {
                        "primitiveType": "DOUBLE",
                    },
                },
                {
                    "display_name": "PII type",
                    "fieldId": "pii_type",
                    "type": {
                        "enumType": {
                            "allowedValues": [
                                {
                                    "display_name": "EMAIL",
                                },
                                {
                                    "display_name": "SOCIAL SECURITY NUMBER",
                                },
                                {
                                    "display_name": "NONE",
                                },
                            ],
                        },
                    },
                },
            ],
            force_delete="false",
            region="us-central1",
            tag_template_id="my_template")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] display_name: The display name for this template.
        :param pulumi.Input[list] fields: Set of tag template field IDs and the settings for the field. This set is an exhaustive list of the allowed fields. This set must contain at least one field and at most 500 fields.  Structure is documented below.
        :param pulumi.Input[bool] force_delete: This confirms the deletion of any possible tags using this template. Must be set to true in order to delete the tag template.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: Template location region.
        :param pulumi.Input[str] tag_template_id: The id of the tag template to create.

        The **fields** object supports the following:

          * `display_name` (`pulumi.Input[str]`) - The display name for this template.
          * `fieldId` (`pulumi.Input[str]`) - The identifier for this object. Format specified above.
          * `isRequired` (`pulumi.Input[bool]`) - Whether this is a required field. Defaults to false.
          * `name` (`pulumi.Input[str]`) - -
            The resource name of the tag template field in URL format. Example: projects/{project_id}/locations/{location}/tagTemplates/{tagTemplateId}/fields/{field}
          * `order` (`pulumi.Input[float]`) - The order of this field with respect to other fields in this tag template.
            A higher value indicates a more important field. The value can be negative.
            Multiple fields can have the same order, and field orders within a tag do not have to be sequential.
          * `type` (`pulumi.Input[dict]`) - The type of value this tag field can contain.  Structure is documented below.
            * `enumType` (`pulumi.Input[dict]`) - Represents an enum type.
              Exactly one of `primitive_type` or `enum_type` must be set  Structure is documented below.
              * `allowedValues` (`pulumi.Input[list]`) - The set of allowed values for this enum. The display names of the
                values must be case-insensitively unique within this set. Currently,
                enum values can only be added to the list of allowed values. Deletion
                and renaming of enum values are not supported.
                Can have up to 500 allowed values.  Structure is documented below.
                * `display_name` (`pulumi.Input[str]`) - The display name for this template.

            * `primitiveType` (`pulumi.Input[str]`) - Represents primitive types - string, bool etc.
              Exactly one of `primitive_type` or `enum_type` must be set
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['display_name'] = display_name
            if fields is None:
                raise TypeError("Missing required property 'fields'")
            __props__['fields'] = fields
            __props__['force_delete'] = force_delete
            __props__['project'] = project
            __props__['region'] = region
            if tag_template_id is None:
                raise TypeError("Missing required property 'tag_template_id'")
            __props__['tag_template_id'] = tag_template_id
            __props__['name'] = None
        super(TagTemplate, __self__).__init__(
            'gcp:datacatalog/tagTemplate:TagTemplate',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, display_name=None, fields=None, force_delete=None, name=None, project=None, region=None, tag_template_id=None):
        """
        Get an existing TagTemplate resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] display_name: The display name for this template.
        :param pulumi.Input[list] fields: Set of tag template field IDs and the settings for the field. This set is an exhaustive list of the allowed fields. This set must contain at least one field and at most 500 fields.  Structure is documented below.
        :param pulumi.Input[bool] force_delete: This confirms the deletion of any possible tags using this template. Must be set to true in order to delete the tag template.
        :param pulumi.Input[str] name: -
               The resource name of the tag template field in URL format. Example: projects/{project_id}/locations/{location}/tagTemplates/{tagTemplateId}/fields/{field}
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: Template location region.
        :param pulumi.Input[str] tag_template_id: The id of the tag template to create.

        The **fields** object supports the following:

          * `display_name` (`pulumi.Input[str]`) - The display name for this template.
          * `fieldId` (`pulumi.Input[str]`) - The identifier for this object. Format specified above.
          * `isRequired` (`pulumi.Input[bool]`) - Whether this is a required field. Defaults to false.
          * `name` (`pulumi.Input[str]`) - -
            The resource name of the tag template field in URL format. Example: projects/{project_id}/locations/{location}/tagTemplates/{tagTemplateId}/fields/{field}
          * `order` (`pulumi.Input[float]`) - The order of this field with respect to other fields in this tag template.
            A higher value indicates a more important field. The value can be negative.
            Multiple fields can have the same order, and field orders within a tag do not have to be sequential.
          * `type` (`pulumi.Input[dict]`) - The type of value this tag field can contain.  Structure is documented below.
            * `enumType` (`pulumi.Input[dict]`) - Represents an enum type.
              Exactly one of `primitive_type` or `enum_type` must be set  Structure is documented below.
              * `allowedValues` (`pulumi.Input[list]`) - The set of allowed values for this enum. The display names of the
                values must be case-insensitively unique within this set. Currently,
                enum values can only be added to the list of allowed values. Deletion
                and renaming of enum values are not supported.
                Can have up to 500 allowed values.  Structure is documented below.
                * `display_name` (`pulumi.Input[str]`) - The display name for this template.

            * `primitiveType` (`pulumi.Input[str]`) - Represents primitive types - string, bool etc.
              Exactly one of `primitive_type` or `enum_type` must be set
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["display_name"] = display_name
        __props__["fields"] = fields
        __props__["force_delete"] = force_delete
        __props__["name"] = name
        __props__["project"] = project
        __props__["region"] = region
        __props__["tag_template_id"] = tag_template_id
        return TagTemplate(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Service(pulumi.CustomResource):
    autogenerate_revision_name: pulumi.Output[bool]
    """
    If set to `true`, the revision name (template.metadata.name) will be omitted and 
    autogenerated by Cloud Run. This cannot be set to `true` while `template.metadata.name`
    is also set.
    (For legacy support, if `template.metadata.name` is unset in state while
    this field is set to false, the revision name will still autogenerate.)
    """
    location: pulumi.Output[str]
    """
    The location of the cloud run instance. eg us-central1
    """
    metadata: pulumi.Output[dict]
    """
    Metadata associated with this Service, including name, namespace, labels,
    and annotations.  Structure is documented below.

      * `annotations` (`dict`) - Annotations is a key value map stored with a resource that
        may be set by external tools to store and retrieve arbitrary metadata. More
        info: http://kubernetes.io/docs/user-guide/annotations
      * `generation` (`float`) - -
        A sequence number representing a specific generation of the desired state.
      * `labels` (`dict`) - Map of string keys and values that can be used to organize and categorize
        (scope and select) objects. May match selectors of replication controllers
        and routes.
        More info: http://kubernetes.io/docs/user-guide/labels
      * `namespace` (`str`) - In Cloud Run the namespace must be equal to either the
        project ID or project number.
      * `resourceVersion` (`str`) - -
        An opaque value that represents the internal version of this object that
        can be used by clients to determine when objects have changed. May be used
        for optimistic concurrency, change detection, and the watch operation on a
        resource or set of resources. They may only be valid for a
        particular resource or set of resources.
        More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
      * `self_link` (`str`) - -
        SelfLink is a URL representing this object.
      * `uid` (`str`) - -
        UID is a unique id generated by the server on successful creation of a resource and is not
        allowed to change on PUT operations.
        More info: http://kubernetes.io/docs/user-guide/identifiers#uids
    """
    name: pulumi.Output[str]
    """
    Name of the environment variable.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    status: pulumi.Output[dict]
    """
    The current status of the Service.

      * `conditions` (`list`)
        * `message` (`str`)
        * `reason` (`str`)
        * `status` (`str`)
        * `type` (`str`)

      * `latestCreatedRevisionName` (`str`)
      * `latestReadyRevisionName` (`str`)
      * `observedGeneration` (`float`)
      * `url` (`str`)
    """
    template: pulumi.Output[dict]
    """
    template holds the latest specification for the Revision to
    be stamped out. The template references the container image, and may also
    include labels and annotations that should be attached to the Revision.
    To correlate a Revision, and/or to force a Revision to be created when the
    spec doesn't otherwise change, a nonce label may be provided in the
    template metadata. For more details, see:
    https://github.com/knative/serving/blob/master/docs/client-conventions.md#associate-modifications-with-revisions
    Cloud Run does not currently support referencing a build that is
    responsible for materializing the container image from source.  Structure is documented below.

      * `metadata` (`dict`) - Metadata associated with this Service, including name, namespace, labels,
        and annotations.  Structure is documented below.
        * `annotations` (`dict`) - Annotations is a key value map stored with a resource that
          may be set by external tools to store and retrieve arbitrary metadata. More
          info: http://kubernetes.io/docs/user-guide/annotations
        * `generation` (`float`) - -
          A sequence number representing a specific generation of the desired state.
        * `labels` (`dict`) - Map of string keys and values that can be used to organize and categorize
          (scope and select) objects. May match selectors of replication controllers
          and routes.
          More info: http://kubernetes.io/docs/user-guide/labels
        * `name` (`str`) - Name of the environment variable.
        * `namespace` (`str`) - In Cloud Run the namespace must be equal to either the
          project ID or project number.
        * `resourceVersion` (`str`) - -
          An opaque value that represents the internal version of this object that
          can be used by clients to determine when objects have changed. May be used
          for optimistic concurrency, change detection, and the watch operation on a
          resource or set of resources. They may only be valid for a
          particular resource or set of resources.
          More info:
          https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
        * `self_link` (`str`) - -
          SelfLink is a URL representing this object.
        * `uid` (`str`) - -
          UID is a unique id generated by the server on successful creation of a resource and is not
          allowed to change on PUT operations.
          More info: http://kubernetes.io/docs/user-guide/identifiers#uids

      * `spec` (`dict`) - RevisionSpec holds the desired state of the Revision (from the client).  Structure is documented below.
        * `containerConcurrency` (`float`) - ContainerConcurrency specifies the maximum allowed in-flight (concurrent)
          requests per container of the Revision. Values are:
          - `0` thread-safe, the system should manage the max concurrency. This is
          the default value.
          - `1` not-thread-safe. Single concurrency
          - `2-N` thread-safe, max concurrency of N
        * `containers` (`list`) - Container defines the unit of execution for this Revision.
          In the context of a Revision, we disallow a number of the fields of
          this Container, including: name, ports, and volumeMounts.
          The runtime contract is documented here:
          https://github.com/knative/serving/blob/master/docs/runtime-contract.md  Structure is documented below.
          * `args` (`list`) - Arguments to the entrypoint.
            The docker image's CMD is used if this is not provided.
            Variable references $(VAR_NAME) are expanded using the container's
            environment. If a variable cannot be resolved, the reference in the input
            string will be unchanged. The $(VAR_NAME) syntax can be escaped with a
            double $$, ie: $$(VAR_NAME). Escaped references will never be expanded,
            regardless of whether the variable exists or not.
            More info:
            https://kubernetes.io/docs/tasks/inject-data-application/define-command-argument-container/#running-a-command-in-a-shell
          * `commands` (`list`) - Entrypoint array. Not executed within a shell.
            The docker image's ENTRYPOINT is used if this is not provided.
            Variable references $(VAR_NAME) are expanded using the container's
            environment. If a variable cannot be resolved, the reference in the input
            string will be unchanged. The $(VAR_NAME) syntax can be escaped with a
            double $$, ie: $$(VAR_NAME). Escaped references will never be expanded,
            regardless of whether the variable exists or not.
            More info:
            https://kubernetes.io/docs/tasks/inject-data-application/define-command-argument-container/#running-a-command-in-a-shell
          * `envFroms` (`list`) - -
            (Optional, Deprecated)
            List of sources to populate environment variables in the container.
            All invalid keys will be reported as an event when the container is starting.
            When a key exists in multiple sources, the value associated with the last source will
            take precedence. Values defined by an Env with a duplicate key will take
            precedence.  Structure is documented below.
            * `configMapRef` (`dict`) - The ConfigMap to select from.  Structure is documented below.
              * `localObjectReference` (`dict`) - The Secret to select from.  Structure is documented below.
                * `name` (`str`) - Name of the environment variable.

              * `optional` (`bool`) - Specify whether the Secret must be defined

            * `prefix` (`str`) - An optional identifier to prepend to each key in the ConfigMap.
            * `secretRef` (`dict`) - The Secret to select from.  Structure is documented below.
              * `localObjectReference` (`dict`) - The Secret to select from.  Structure is documented below.
                * `name` (`str`) - Name of the environment variable.

              * `optional` (`bool`) - Specify whether the Secret must be defined

          * `envs` (`list`) - List of environment variables to set in the container.  Structure is documented below.
            * `name` (`str`) - Name of the environment variable.
            * `value` (`str`) - Variable references $(VAR_NAME) are expanded
              using the previous defined environment variables in the container and
              any route environment variables. If a variable cannot be resolved,
              the reference in the input string will be unchanged. The $(VAR_NAME)
              syntax can be escaped with a double $$, ie: $$(VAR_NAME). Escaped
              references will never be expanded, regardless of whether the variable
              exists or not.
              Defaults to "".

          * `image` (`str`) - Docker image name. This is most often a reference to a container located
            in the container registry, such as gcr.io/cloudrun/hello
            More info: https://kubernetes.io/docs/concepts/containers/images
          * `resources` (`dict`) - Compute Resources required by this container. Used to set values such as max memory
            More info:
            https://kubernetes.io/docs/concepts/storage/persistent-volumes#resources  Structure is documented below.
            * `limits` (`dict`) - Limits describes the maximum amount of compute resources allowed.
              The values of the map is string form of the 'quantity' k8s type:
              https://github.com/kubernetes/kubernetes/blob/master/staging/src/k8s.io/apimachinery/pkg/api/resource/quantity.go
            * `requests` (`dict`) - Requests describes the minimum amount of compute resources required.
              If Requests is omitted for a container, it defaults to Limits if that is
              explicitly specified, otherwise to an implementation-defined value.
              The values of the map is string form of the 'quantity' k8s type:
              https://github.com/kubernetes/kubernetes/blob/master/staging/src/k8s.io/apimachinery/pkg/api/resource/quantity.go

          * `workingDir` (`str`) - -
            (Optional, Deprecated)
            Container's working directory.
            If not specified, the container runtime's default will be used, which
            might be configured in the container image.

        * `service_account_name` (`str`) - Email address of the IAM service account associated with the revision of the
          service. The service account represents the identity of the running revision,
          and determines what permissions the revision has. If not provided, the revision
          will use the project's default service account.
        * `servingState` (`str`) - -
          ServingState holds a value describing the state the resources
          are in for this Revision.
          It is expected
          that the system will manipulate this based on routability and load.
    """
    traffics: pulumi.Output[list]
    """
    Traffic specifies how to distribute traffic over a collection of Knative Revisions
    and Configurations  Structure is documented below.

      * `latestRevision` (`bool`) - LatestRevision may be optionally provided to indicate that the latest ready
        Revision of the Configuration should be used for this traffic target. When
        provided LatestRevision must be true if RevisionName is empty; it must be
        false when RevisionName is non-empty.
      * `percent` (`float`) - Percent specifies percent of the traffic to this Revision or Configuration.
      * `revisionName` (`str`) - RevisionName of a specific revision to which to send this portion of traffic.
    """
    def __init__(__self__, resource_name, opts=None, autogenerate_revision_name=None, location=None, metadata=None, name=None, project=None, template=None, traffics=None, __props__=None, __name__=None, __opts__=None):
        """
        Service acts as a top-level container that manages a set of Routes and
        Configurations which implement a network service. Service exists to provide a
        singular abstraction which can be access controlled, reasoned about, and
        which encapsulates software lifecycle decisions such as rollout policy and
        team resource ownership. Service acts only as an orchestrator of the
        underlying Routes and Configurations (much as a kubernetes Deployment
        orchestrates ReplicaSets).

        The Service's controller will track the statuses of its owned Configuration
        and Route, reflecting their statuses and conditions as its own.

        See also:
        https://github.com/knative/serving/blob/master/docs/spec/overview.md#service


        To get more information about Service, see:

        * [API documentation](https://cloud.google.com/run/docs/reference/rest/v1/projects.locations.services)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/run/docs/)


        ## Example Usage

        ### Cloud Run Service Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.cloudrun.Service("default",
            location="us-central1",
            template={
                "spec": {
                    "containers": [{
                        "image": "gcr.io/cloudrun/hello",
                    }],
                },
            },
            traffics=[{
                "latestRevision": True,
                "percent": 100,
            }])
        ```

        ### Cloud Run Service Sql

        ```python
        import pulumi
        import pulumi_gcp as gcp

        instance = gcp.sql.DatabaseInstance("instance",
            region="us-east1",
            settings={
                "tier": "db-f1-micro",
            })
        default = gcp.cloudrun.Service("default",
            autogenerate_revision_name=True,
            location="us-central1",
            template={
                "metadata": {
                    "annotations": {
                        "autoscaling.knative.dev/maxScale": "1000",
                        "run.googleapis.com/client-name": "demo",
                        "run.googleapis.com/cloudsql-instances": instance.name.apply(lambda name: f"my-project-name:us-central1:{name}"),
                    },
                },
                "spec": {
                    "containers": [{
                        "image": "gcr.io/cloudrun/hello",
                    }],
                },
            })
        ```

        ### Cloud Run Service Multiple Environment Variables

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.cloudrun.Service("default",
            autogenerate_revision_name=True,
            location="us-central1",
            template={
                "spec": {
                    "containers": [{
                        "env": [
                            {
                                "name": "SOURCE",
                                "value": "remote",
                            },
                            {
                                "name": "TARGET",
                                "value": "home",
                            },
                        ],
                        "image": "gcr.io/cloudrun/hello",
                    }],
                },
            },
            traffics=[{
                "latestRevision": True,
                "percent": 100,
            }])
        ```

        ### Cloud Run Service Traffic Split

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.cloudrun.Service("default",
            location="us-central1",
            template={
                "metadata": {
                    "name": "cloudrun-srv-green",
                },
                "spec": {
                    "containers": [{
                        "image": "gcr.io/cloudrun/hello",
                    }],
                },
            },
            traffics=[
                {
                    "percent": 25,
                    "revisionName": "cloudrun-srv-green",
                },
                {
                    "percent": 75,
                    "revisionName": "cloudrun-srv-blue",
                },
            ])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] autogenerate_revision_name: If set to `true`, the revision name (template.metadata.name) will be omitted and 
               autogenerated by Cloud Run. This cannot be set to `true` while `template.metadata.name`
               is also set.
               (For legacy support, if `template.metadata.name` is unset in state while
               this field is set to false, the revision name will still autogenerate.)
        :param pulumi.Input[str] location: The location of the cloud run instance. eg us-central1
        :param pulumi.Input[dict] metadata: Metadata associated with this Service, including name, namespace, labels,
               and annotations.  Structure is documented below.
        :param pulumi.Input[str] name: Name of the environment variable.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] template: template holds the latest specification for the Revision to
               be stamped out. The template references the container image, and may also
               include labels and annotations that should be attached to the Revision.
               To correlate a Revision, and/or to force a Revision to be created when the
               spec doesn't otherwise change, a nonce label may be provided in the
               template metadata. For more details, see:
               https://github.com/knative/serving/blob/master/docs/client-conventions.md#associate-modifications-with-revisions
               Cloud Run does not currently support referencing a build that is
               responsible for materializing the container image from source.  Structure is documented below.
        :param pulumi.Input[list] traffics: Traffic specifies how to distribute traffic over a collection of Knative Revisions
               and Configurations  Structure is documented below.

        The **metadata** object supports the following:

          * `annotations` (`pulumi.Input[dict]`) - Annotations is a key value map stored with a resource that
            may be set by external tools to store and retrieve arbitrary metadata. More
            info: http://kubernetes.io/docs/user-guide/annotations
          * `generation` (`pulumi.Input[float]`) - -
            A sequence number representing a specific generation of the desired state.
          * `labels` (`pulumi.Input[dict]`) - Map of string keys and values that can be used to organize and categorize
            (scope and select) objects. May match selectors of replication controllers
            and routes.
            More info: http://kubernetes.io/docs/user-guide/labels
          * `namespace` (`pulumi.Input[str]`) - In Cloud Run the namespace must be equal to either the
            project ID or project number.
          * `resourceVersion` (`pulumi.Input[str]`) - -
            An opaque value that represents the internal version of this object that
            can be used by clients to determine when objects have changed. May be used
            for optimistic concurrency, change detection, and the watch operation on a
            resource or set of resources. They may only be valid for a
            particular resource or set of resources.
            More info:
            https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
          * `self_link` (`pulumi.Input[str]`) - -
            SelfLink is a URL representing this object.
          * `uid` (`pulumi.Input[str]`) - -
            UID is a unique id generated by the server on successful creation of a resource and is not
            allowed to change on PUT operations.
            More info: http://kubernetes.io/docs/user-guide/identifiers#uids

        The **template** object supports the following:

          * `metadata` (`pulumi.Input[dict]`) - Metadata associated with this Service, including name, namespace, labels,
            and annotations.  Structure is documented below.
            * `annotations` (`pulumi.Input[dict]`) - Annotations is a key value map stored with a resource that
              may be set by external tools to store and retrieve arbitrary metadata. More
              info: http://kubernetes.io/docs/user-guide/annotations
            * `generation` (`pulumi.Input[float]`) - -
              A sequence number representing a specific generation of the desired state.
            * `labels` (`pulumi.Input[dict]`) - Map of string keys and values that can be used to organize and categorize
              (scope and select) objects. May match selectors of replication controllers
              and routes.
              More info: http://kubernetes.io/docs/user-guide/labels
            * `name` (`pulumi.Input[str]`) - Name of the environment variable.
            * `namespace` (`pulumi.Input[str]`) - In Cloud Run the namespace must be equal to either the
              project ID or project number.
            * `resourceVersion` (`pulumi.Input[str]`) - -
              An opaque value that represents the internal version of this object that
              can be used by clients to determine when objects have changed. May be used
              for optimistic concurrency, change detection, and the watch operation on a
              resource or set of resources. They may only be valid for a
              particular resource or set of resources.
              More info:
              https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
            * `self_link` (`pulumi.Input[str]`) - -
              SelfLink is a URL representing this object.
            * `uid` (`pulumi.Input[str]`) - -
              UID is a unique id generated by the server on successful creation of a resource and is not
              allowed to change on PUT operations.
              More info: http://kubernetes.io/docs/user-guide/identifiers#uids

          * `spec` (`pulumi.Input[dict]`) - RevisionSpec holds the desired state of the Revision (from the client).  Structure is documented below.
            * `containerConcurrency` (`pulumi.Input[float]`) - ContainerConcurrency specifies the maximum allowed in-flight (concurrent)
              requests per container of the Revision. Values are:
              - `0` thread-safe, the system should manage the max concurrency. This is
              the default value.
              - `1` not-thread-safe. Single concurrency
              - `2-N` thread-safe, max concurrency of N
            * `containers` (`pulumi.Input[list]`) - Container defines the unit of execution for this Revision.
              In the context of a Revision, we disallow a number of the fields of
              this Container, including: name, ports, and volumeMounts.
              The runtime contract is documented here:
              https://github.com/knative/serving/blob/master/docs/runtime-contract.md  Structure is documented below.
              * `args` (`pulumi.Input[list]`) - Arguments to the entrypoint.
                The docker image's CMD is used if this is not provided.
                Variable references $(VAR_NAME) are expanded using the container's
                environment. If a variable cannot be resolved, the reference in the input
                string will be unchanged. The $(VAR_NAME) syntax can be escaped with a
                double $$, ie: $$(VAR_NAME). Escaped references will never be expanded,
                regardless of whether the variable exists or not.
                More info:
                https://kubernetes.io/docs/tasks/inject-data-application/define-command-argument-container/#running-a-command-in-a-shell
              * `commands` (`pulumi.Input[list]`) - Entrypoint array. Not executed within a shell.
                The docker image's ENTRYPOINT is used if this is not provided.
                Variable references $(VAR_NAME) are expanded using the container's
                environment. If a variable cannot be resolved, the reference in the input
                string will be unchanged. The $(VAR_NAME) syntax can be escaped with a
                double $$, ie: $$(VAR_NAME). Escaped references will never be expanded,
                regardless of whether the variable exists or not.
                More info:
                https://kubernetes.io/docs/tasks/inject-data-application/define-command-argument-container/#running-a-command-in-a-shell
              * `envFroms` (`pulumi.Input[list]`) - -
                (Optional, Deprecated)
                List of sources to populate environment variables in the container.
                All invalid keys will be reported as an event when the container is starting.
                When a key exists in multiple sources, the value associated with the last source will
                take precedence. Values defined by an Env with a duplicate key will take
                precedence.  Structure is documented below.
                * `configMapRef` (`pulumi.Input[dict]`) - The ConfigMap to select from.  Structure is documented below.
                  * `localObjectReference` (`pulumi.Input[dict]`) - The Secret to select from.  Structure is documented below.
                    * `name` (`pulumi.Input[str]`) - Name of the environment variable.

                  * `optional` (`pulumi.Input[bool]`) - Specify whether the Secret must be defined

                * `prefix` (`pulumi.Input[str]`) - An optional identifier to prepend to each key in the ConfigMap.
                * `secretRef` (`pulumi.Input[dict]`) - The Secret to select from.  Structure is documented below.
                  * `localObjectReference` (`pulumi.Input[dict]`) - The Secret to select from.  Structure is documented below.
                    * `name` (`pulumi.Input[str]`) - Name of the environment variable.

                  * `optional` (`pulumi.Input[bool]`) - Specify whether the Secret must be defined

              * `envs` (`pulumi.Input[list]`) - List of environment variables to set in the container.  Structure is documented below.
                * `name` (`pulumi.Input[str]`) - Name of the environment variable.
                * `value` (`pulumi.Input[str]`) - Variable references $(VAR_NAME) are expanded
                  using the previous defined environment variables in the container and
                  any route environment variables. If a variable cannot be resolved,
                  the reference in the input string will be unchanged. The $(VAR_NAME)
                  syntax can be escaped with a double $$, ie: $$(VAR_NAME). Escaped
                  references will never be expanded, regardless of whether the variable
                  exists or not.
                  Defaults to "".

              * `image` (`pulumi.Input[str]`) - Docker image name. This is most often a reference to a container located
                in the container registry, such as gcr.io/cloudrun/hello
                More info: https://kubernetes.io/docs/concepts/containers/images
              * `resources` (`pulumi.Input[dict]`) - Compute Resources required by this container. Used to set values such as max memory
                More info:
                https://kubernetes.io/docs/concepts/storage/persistent-volumes#resources  Structure is documented below.
                * `limits` (`pulumi.Input[dict]`) - Limits describes the maximum amount of compute resources allowed.
                  The values of the map is string form of the 'quantity' k8s type:
                  https://github.com/kubernetes/kubernetes/blob/master/staging/src/k8s.io/apimachinery/pkg/api/resource/quantity.go
                * `requests` (`pulumi.Input[dict]`) - Requests describes the minimum amount of compute resources required.
                  If Requests is omitted for a container, it defaults to Limits if that is
                  explicitly specified, otherwise to an implementation-defined value.
                  The values of the map is string form of the 'quantity' k8s type:
                  https://github.com/kubernetes/kubernetes/blob/master/staging/src/k8s.io/apimachinery/pkg/api/resource/quantity.go

              * `workingDir` (`pulumi.Input[str]`) - -
                (Optional, Deprecated)
                Container's working directory.
                If not specified, the container runtime's default will be used, which
                might be configured in the container image.

            * `service_account_name` (`pulumi.Input[str]`) - Email address of the IAM service account associated with the revision of the
              service. The service account represents the identity of the running revision,
              and determines what permissions the revision has. If not provided, the revision
              will use the project's default service account.
            * `servingState` (`pulumi.Input[str]`) - -
              ServingState holds a value describing the state the resources
              are in for this Revision.
              It is expected
              that the system will manipulate this based on routability and load.

        The **traffics** object supports the following:

          * `latestRevision` (`pulumi.Input[bool]`) - LatestRevision may be optionally provided to indicate that the latest ready
            Revision of the Configuration should be used for this traffic target. When
            provided LatestRevision must be true if RevisionName is empty; it must be
            false when RevisionName is non-empty.
          * `percent` (`pulumi.Input[float]`) - Percent specifies percent of the traffic to this Revision or Configuration.
          * `revisionName` (`pulumi.Input[str]`) - RevisionName of a specific revision to which to send this portion of traffic.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['autogenerate_revision_name'] = autogenerate_revision_name
            if location is None:
                raise TypeError("Missing required property 'location'")
            __props__['location'] = location
            __props__['metadata'] = metadata
            __props__['name'] = name
            __props__['project'] = project
            __props__['template'] = template
            __props__['traffics'] = traffics
            __props__['status'] = None
        super(Service, __self__).__init__(
            'gcp:cloudrun/service:Service',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, autogenerate_revision_name=None, location=None, metadata=None, name=None, project=None, status=None, template=None, traffics=None):
        """
        Get an existing Service resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] autogenerate_revision_name: If set to `true`, the revision name (template.metadata.name) will be omitted and 
               autogenerated by Cloud Run. This cannot be set to `true` while `template.metadata.name`
               is also set.
               (For legacy support, if `template.metadata.name` is unset in state while
               this field is set to false, the revision name will still autogenerate.)
        :param pulumi.Input[str] location: The location of the cloud run instance. eg us-central1
        :param pulumi.Input[dict] metadata: Metadata associated with this Service, including name, namespace, labels,
               and annotations.  Structure is documented below.
        :param pulumi.Input[str] name: Name of the environment variable.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] status: The current status of the Service.
        :param pulumi.Input[dict] template: template holds the latest specification for the Revision to
               be stamped out. The template references the container image, and may also
               include labels and annotations that should be attached to the Revision.
               To correlate a Revision, and/or to force a Revision to be created when the
               spec doesn't otherwise change, a nonce label may be provided in the
               template metadata. For more details, see:
               https://github.com/knative/serving/blob/master/docs/client-conventions.md#associate-modifications-with-revisions
               Cloud Run does not currently support referencing a build that is
               responsible for materializing the container image from source.  Structure is documented below.
        :param pulumi.Input[list] traffics: Traffic specifies how to distribute traffic over a collection of Knative Revisions
               and Configurations  Structure is documented below.

        The **metadata** object supports the following:

          * `annotations` (`pulumi.Input[dict]`) - Annotations is a key value map stored with a resource that
            may be set by external tools to store and retrieve arbitrary metadata. More
            info: http://kubernetes.io/docs/user-guide/annotations
          * `generation` (`pulumi.Input[float]`) - -
            A sequence number representing a specific generation of the desired state.
          * `labels` (`pulumi.Input[dict]`) - Map of string keys and values that can be used to organize and categorize
            (scope and select) objects. May match selectors of replication controllers
            and routes.
            More info: http://kubernetes.io/docs/user-guide/labels
          * `namespace` (`pulumi.Input[str]`) - In Cloud Run the namespace must be equal to either the
            project ID or project number.
          * `resourceVersion` (`pulumi.Input[str]`) - -
            An opaque value that represents the internal version of this object that
            can be used by clients to determine when objects have changed. May be used
            for optimistic concurrency, change detection, and the watch operation on a
            resource or set of resources. They may only be valid for a
            particular resource or set of resources.
            More info:
            https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
          * `self_link` (`pulumi.Input[str]`) - -
            SelfLink is a URL representing this object.
          * `uid` (`pulumi.Input[str]`) - -
            UID is a unique id generated by the server on successful creation of a resource and is not
            allowed to change on PUT operations.
            More info: http://kubernetes.io/docs/user-guide/identifiers#uids

        The **status** object supports the following:

          * `conditions` (`pulumi.Input[list]`)
            * `message` (`pulumi.Input[str]`)
            * `reason` (`pulumi.Input[str]`)
            * `status` (`pulumi.Input[str]`)
            * `type` (`pulumi.Input[str]`)

          * `latestCreatedRevisionName` (`pulumi.Input[str]`)
          * `latestReadyRevisionName` (`pulumi.Input[str]`)
          * `observedGeneration` (`pulumi.Input[float]`)
          * `url` (`pulumi.Input[str]`)

        The **template** object supports the following:

          * `metadata` (`pulumi.Input[dict]`) - Metadata associated with this Service, including name, namespace, labels,
            and annotations.  Structure is documented below.
            * `annotations` (`pulumi.Input[dict]`) - Annotations is a key value map stored with a resource that
              may be set by external tools to store and retrieve arbitrary metadata. More
              info: http://kubernetes.io/docs/user-guide/annotations
            * `generation` (`pulumi.Input[float]`) - -
              A sequence number representing a specific generation of the desired state.
            * `labels` (`pulumi.Input[dict]`) - Map of string keys and values that can be used to organize and categorize
              (scope and select) objects. May match selectors of replication controllers
              and routes.
              More info: http://kubernetes.io/docs/user-guide/labels
            * `name` (`pulumi.Input[str]`) - Name of the environment variable.
            * `namespace` (`pulumi.Input[str]`) - In Cloud Run the namespace must be equal to either the
              project ID or project number.
            * `resourceVersion` (`pulumi.Input[str]`) - -
              An opaque value that represents the internal version of this object that
              can be used by clients to determine when objects have changed. May be used
              for optimistic concurrency, change detection, and the watch operation on a
              resource or set of resources. They may only be valid for a
              particular resource or set of resources.
              More info:
              https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
            * `self_link` (`pulumi.Input[str]`) - -
              SelfLink is a URL representing this object.
            * `uid` (`pulumi.Input[str]`) - -
              UID is a unique id generated by the server on successful creation of a resource and is not
              allowed to change on PUT operations.
              More info: http://kubernetes.io/docs/user-guide/identifiers#uids

          * `spec` (`pulumi.Input[dict]`) - RevisionSpec holds the desired state of the Revision (from the client).  Structure is documented below.
            * `containerConcurrency` (`pulumi.Input[float]`) - ContainerConcurrency specifies the maximum allowed in-flight (concurrent)
              requests per container of the Revision. Values are:
              - `0` thread-safe, the system should manage the max concurrency. This is
              the default value.
              - `1` not-thread-safe. Single concurrency
              - `2-N` thread-safe, max concurrency of N
            * `containers` (`pulumi.Input[list]`) - Container defines the unit of execution for this Revision.
              In the context of a Revision, we disallow a number of the fields of
              this Container, including: name, ports, and volumeMounts.
              The runtime contract is documented here:
              https://github.com/knative/serving/blob/master/docs/runtime-contract.md  Structure is documented below.
              * `args` (`pulumi.Input[list]`) - Arguments to the entrypoint.
                The docker image's CMD is used if this is not provided.
                Variable references $(VAR_NAME) are expanded using the container's
                environment. If a variable cannot be resolved, the reference in the input
                string will be unchanged. The $(VAR_NAME) syntax can be escaped with a
                double $$, ie: $$(VAR_NAME). Escaped references will never be expanded,
                regardless of whether the variable exists or not.
                More info:
                https://kubernetes.io/docs/tasks/inject-data-application/define-command-argument-container/#running-a-command-in-a-shell
              * `commands` (`pulumi.Input[list]`) - Entrypoint array. Not executed within a shell.
                The docker image's ENTRYPOINT is used if this is not provided.
                Variable references $(VAR_NAME) are expanded using the container's
                environment. If a variable cannot be resolved, the reference in the input
                string will be unchanged. The $(VAR_NAME) syntax can be escaped with a
                double $$, ie: $$(VAR_NAME). Escaped references will never be expanded,
                regardless of whether the variable exists or not.
                More info:
                https://kubernetes.io/docs/tasks/inject-data-application/define-command-argument-container/#running-a-command-in-a-shell
              * `envFroms` (`pulumi.Input[list]`) - -
                (Optional, Deprecated)
                List of sources to populate environment variables in the container.
                All invalid keys will be reported as an event when the container is starting.
                When a key exists in multiple sources, the value associated with the last source will
                take precedence. Values defined by an Env with a duplicate key will take
                precedence.  Structure is documented below.
                * `configMapRef` (`pulumi.Input[dict]`) - The ConfigMap to select from.  Structure is documented below.
                  * `localObjectReference` (`pulumi.Input[dict]`) - The Secret to select from.  Structure is documented below.
                    * `name` (`pulumi.Input[str]`) - Name of the environment variable.

                  * `optional` (`pulumi.Input[bool]`) - Specify whether the Secret must be defined

                * `prefix` (`pulumi.Input[str]`) - An optional identifier to prepend to each key in the ConfigMap.
                * `secretRef` (`pulumi.Input[dict]`) - The Secret to select from.  Structure is documented below.
                  * `localObjectReference` (`pulumi.Input[dict]`) - The Secret to select from.  Structure is documented below.
                    * `name` (`pulumi.Input[str]`) - Name of the environment variable.

                  * `optional` (`pulumi.Input[bool]`) - Specify whether the Secret must be defined

              * `envs` (`pulumi.Input[list]`) - List of environment variables to set in the container.  Structure is documented below.
                * `name` (`pulumi.Input[str]`) - Name of the environment variable.
                * `value` (`pulumi.Input[str]`) - Variable references $(VAR_NAME) are expanded
                  using the previous defined environment variables in the container and
                  any route environment variables. If a variable cannot be resolved,
                  the reference in the input string will be unchanged. The $(VAR_NAME)
                  syntax can be escaped with a double $$, ie: $$(VAR_NAME). Escaped
                  references will never be expanded, regardless of whether the variable
                  exists or not.
                  Defaults to "".

              * `image` (`pulumi.Input[str]`) - Docker image name. This is most often a reference to a container located
                in the container registry, such as gcr.io/cloudrun/hello
                More info: https://kubernetes.io/docs/concepts/containers/images
              * `resources` (`pulumi.Input[dict]`) - Compute Resources required by this container. Used to set values such as max memory
                More info:
                https://kubernetes.io/docs/concepts/storage/persistent-volumes#resources  Structure is documented below.
                * `limits` (`pulumi.Input[dict]`) - Limits describes the maximum amount of compute resources allowed.
                  The values of the map is string form of the 'quantity' k8s type:
                  https://github.com/kubernetes/kubernetes/blob/master/staging/src/k8s.io/apimachinery/pkg/api/resource/quantity.go
                * `requests` (`pulumi.Input[dict]`) - Requests describes the minimum amount of compute resources required.
                  If Requests is omitted for a container, it defaults to Limits if that is
                  explicitly specified, otherwise to an implementation-defined value.
                  The values of the map is string form of the 'quantity' k8s type:
                  https://github.com/kubernetes/kubernetes/blob/master/staging/src/k8s.io/apimachinery/pkg/api/resource/quantity.go

              * `workingDir` (`pulumi.Input[str]`) - -
                (Optional, Deprecated)
                Container's working directory.
                If not specified, the container runtime's default will be used, which
                might be configured in the container image.

            * `service_account_name` (`pulumi.Input[str]`) - Email address of the IAM service account associated with the revision of the
              service. The service account represents the identity of the running revision,
              and determines what permissions the revision has. If not provided, the revision
              will use the project's default service account.
            * `servingState` (`pulumi.Input[str]`) - -
              ServingState holds a value describing the state the resources
              are in for this Revision.
              It is expected
              that the system will manipulate this based on routability and load.

        The **traffics** object supports the following:

          * `latestRevision` (`pulumi.Input[bool]`) - LatestRevision may be optionally provided to indicate that the latest ready
            Revision of the Configuration should be used for this traffic target. When
            provided LatestRevision must be true if RevisionName is empty; it must be
            false when RevisionName is non-empty.
          * `percent` (`pulumi.Input[float]`) - Percent specifies percent of the traffic to this Revision or Configuration.
          * `revisionName` (`pulumi.Input[str]`) - RevisionName of a specific revision to which to send this portion of traffic.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["autogenerate_revision_name"] = autogenerate_revision_name
        __props__["location"] = location
        __props__["metadata"] = metadata
        __props__["name"] = name
        __props__["project"] = project
        __props__["status"] = status
        __props__["template"] = template
        __props__["traffics"] = traffics
        return Service(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class IAMBinding(pulumi.CustomResource):
    condition: pulumi.Output[dict]
    """
    An [IAM Condition](https://cloud.google.com/iam/docs/conditions-overview) for a given binding.
    Structure is documented below.

      * `description` (`str`) - An optional description of the expression. This is a longer text which describes the expression, e.g. when hovered over it in a UI.
      * `expression` (`str`) - Textual representation of an expression in Common Expression Language syntax.
      * `title` (`str`) - A title for the expression, i.e. a short string describing its purpose.
    """
    etag: pulumi.Output[str]
    """
    (Computed) The etag of the project's IAM policy.
    """
    members: pulumi.Output[list]
    project: pulumi.Output[str]
    """
    The project ID. If not specified for `projects.IAMBinding`, `projects.IAMMember`, or `projects.IAMAuditConfig`, uses the ID of the project configured with the provider.
    Required for `projects.IAMPolicy` - you must explicitly set the project, and it
    will not be inferred from the provider.
    """
    role: pulumi.Output[str]
    """
    The role that should be applied. Only one
    `projects.IAMBinding` can be used per role. Note that custom roles must be of the format
    `[projects|organizations]/{parent-name}/roles/{role-name}`.
    """
    def __init__(__self__, resource_name, opts=None, condition=None, members=None, project=None, role=None, __props__=None, __name__=None, __opts__=None):
        """
        Four different resources help you manage your IAM policy for a project. Each of these resources serves a different use case:

        * `projects.IAMPolicy`: Authoritative. Sets the IAM policy for the project and replaces any existing policy already attached.
        * `projects.IAMBinding`: Authoritative for a given role. Updates the IAM policy to grant a role to a list of members. Other roles within the IAM policy for the project are preserved.
        * `projects.IAMMember`: Non-authoritative. Updates the IAM policy to grant a role to a new member. Other members for the role for the project are preserved.
        * `projects.IAMAuditConfig`: Authoritative for a given service. Updates the IAM policy to enable audit logging for the given service.


        > **Note:** `projects.IAMPolicy` **cannot** be used in conjunction with `projects.IAMBinding`, `projects.IAMMember`, or `projects.IAMAuditConfig` or they will fight over what your policy should be.

        > **Note:** `projects.IAMBinding` resources **can be** used in conjunction with `projects.IAMMember` resources **only if** they do not grant privilege to the same role.

        ## google\_project\_iam\_policy

        > **Be careful!** You can accidentally lock yourself out of your project
           using this resource. Deleting a `projects.IAMPolicy` removes access
           from anyone without organization-level access to the project. Proceed with caution.
           It's not recommended to use `projects.IAMPolicy` with your provider project
           to avoid locking yourself out, and it should generally only be used with projects
           fully managed by this provider. If you do use this resource, it is recommended to **import** the policy before
           applying the change.

        ```python
        import pulumi
        import pulumi_gcp as gcp

        admin = gcp.organizations.get_iam_policy(binding=[{
            "role": "roles/editor",
            "members": ["user:jane@example.com"],
        }])
        project = gcp.projects.IAMPolicy("project",
            project="your-project-id",
            policy_data=admin.policy_data)
        ```

        With IAM Conditions):

        ```python
        import pulumi
        import pulumi_gcp as gcp

        admin = gcp.organizations.get_iam_policy(bindings=[{
            "condition": {
                "description": "Expiring at midnight of 2019-12-31",
                "expression": "request.time < timestamp(\"2020-01-01T00:00:00Z\")",
                "title": "expires_after_2019_12_31",
            },
            "members": ["user:jane@example.com"],
            "role": "roles/editor",
        }])
        project = gcp.projects.IAMPolicy("project",
            policy_data=admin.policy_data,
            project="your-project-id")
        ```

        ## google\_project\_iam\_binding

        > **Note:** If `role` is set to `roles/owner` and you don't specify a user or service account you have access to in `members`, you can lock yourself out of your project.

        ```python
        import pulumi
        import pulumi_gcp as gcp

        project = gcp.projects.IAMBinding("project",
            members=["user:jane@example.com"],
            project="your-project-id",
            role="roles/editor")
        ```

        With IAM Conditions:

        ```python
        import pulumi
        import pulumi_gcp as gcp

        project = gcp.projects.IAMBinding("project",
            condition={
                "description": "Expiring at midnight of 2019-12-31",
                "expression": "request.time < timestamp(\"2020-01-01T00:00:00Z\")",
                "title": "expires_after_2019_12_31",
            },
            members=["user:jane@example.com"],
            project="your-project-id",
            role="roles/editor")
        ```

        ## google\_project\_iam\_member

        ```python
        import pulumi
        import pulumi_gcp as gcp

        project = gcp.projects.IAMMember("project",
            member="user:jane@example.com",
            project="your-project-id",
            role="roles/editor")
        ```

        With IAM Conditions:

        ```python
        import pulumi
        import pulumi_gcp as gcp

        project = gcp.projects.IAMMember("project",
            condition={
                "description": "Expiring at midnight of 2019-12-31",
                "expression": "request.time < timestamp(\"2020-01-01T00:00:00Z\")",
                "title": "expires_after_2019_12_31",
            },
            member="user:jane@example.com",
            project="your-project-id",
            role="roles/editor")
        ```

        ## google\_project\_iam\_audit\_config

        ```python
        import pulumi
        import pulumi_gcp as gcp

        project = gcp.projects.IAMAuditConfig("project",
            audit_log_configs=[
                {
                    "logType": "ADMIN_READ",
                },
                {
                    "exemptedMembers": ["user:joebloggs@hashicorp.com"],
                    "logType": "DATA_READ",
                },
            ],
            project="your-project-id",
            service="allServices")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] condition: An [IAM Condition](https://cloud.google.com/iam/docs/conditions-overview) for a given binding.
               Structure is documented below.
        :param pulumi.Input[str] project: The project ID. If not specified for `projects.IAMBinding`, `projects.IAMMember`, or `projects.IAMAuditConfig`, uses the ID of the project configured with the provider.
               Required for `projects.IAMPolicy` - you must explicitly set the project, and it
               will not be inferred from the provider.
        :param pulumi.Input[str] role: The role that should be applied. Only one
               `projects.IAMBinding` can be used per role. Note that custom roles must be of the format
               `[projects|organizations]/{parent-name}/roles/{role-name}`.

        The **condition** object supports the following:

          * `description` (`pulumi.Input[str]`) - An optional description of the expression. This is a longer text which describes the expression, e.g. when hovered over it in a UI.
          * `expression` (`pulumi.Input[str]`) - Textual representation of an expression in Common Expression Language syntax.
          * `title` (`pulumi.Input[str]`) - A title for the expression, i.e. a short string describing its purpose.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['condition'] = condition
            if members is None:
                raise TypeError("Missing required property 'members'")
            __props__['members'] = members
            __props__['project'] = project
            if role is None:
                raise TypeError("Missing required property 'role'")
            __props__['role'] = role
            __props__['etag'] = None
        super(IAMBinding, __self__).__init__(
            'gcp:projects/iAMBinding:IAMBinding',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, condition=None, etag=None, members=None, project=None, role=None):
        """
        Get an existing IAMBinding resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] condition: An [IAM Condition](https://cloud.google.com/iam/docs/conditions-overview) for a given binding.
               Structure is documented below.
        :param pulumi.Input[str] etag: (Computed) The etag of the project's IAM policy.
        :param pulumi.Input[str] project: The project ID. If not specified for `projects.IAMBinding`, `projects.IAMMember`, or `projects.IAMAuditConfig`, uses the ID of the project configured with the provider.
               Required for `projects.IAMPolicy` - you must explicitly set the project, and it
               will not be inferred from the provider.
        :param pulumi.Input[str] role: The role that should be applied. Only one
               `projects.IAMBinding` can be used per role. Note that custom roles must be of the format
               `[projects|organizations]/{parent-name}/roles/{role-name}`.

        The **condition** object supports the following:

          * `description` (`pulumi.Input[str]`) - An optional description of the expression. This is a longer text which describes the expression, e.g. when hovered over it in a UI.
          * `expression` (`pulumi.Input[str]`) - Textual representation of an expression in Common Expression Language syntax.
          * `title` (`pulumi.Input[str]`) - A title for the expression, i.e. a short string describing its purpose.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["condition"] = condition
        __props__["etag"] = etag
        __props__["members"] = members
        __props__["project"] = project
        __props__["role"] = role
        return IAMBinding(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

